/* Fo
 * fo-multi-properties.c: Multi Properties formatting object
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-multi-properties.c,v 1.2 2003/03/19 21:11:48 tonygraham Exp $
 *
 * See Copyright for the status of this software.
 */

#include "libfo/fo-utils.h"
#include "fo/fo-fo.h"
#include "fo/fo-fo-private.h"
#include "fo-neutral-fo.h"
#include "fo-multi-properties.h"
#include "fo-multi-properties-private.h"

enum {
  PROP_0,
  PROP_ID,
  PROP_ROLE,
  PROP_SOURCE_DOCUMENT
};

static void fo_multi_properties_init        (FoMultiProperties      *fo_multi_properties);
static void fo_multi_properties_class_init  (FoMultiPropertiesClass *klass);
static void fo_multi_properties_set_property (GObject         *object,
                                  guint            prop_id,
                                  const GValue    *value,
                                  GParamSpec      *pspec);
static void fo_multi_properties_get_property   (GObject         *object,
                                       guint            prop_id,
                                       GValue          *value,
                                       GParamSpec      *pspec);
static void fo_multi_properties_finalize    (GObject           *object);
static void fo_multi_properties_debug_dump_properties (FoFo *fo, gint depth);

static gpointer parent_class;

GType
fo_multi_properties_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoMultiPropertiesClass),
        (GBaseInitFunc) NULL,
        (GBaseFinalizeFunc) NULL,
        (GClassInitFunc) fo_multi_properties_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoMultiProperties),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_multi_properties_init,
      };
      
      static const GInterfaceInfo fo_neutral_fo_info =
      {
        NULL,
        NULL,
        NULL
      };

      object_type = g_type_register_static (FO_TYPE_FO,
                                            "FoMultiProperties",
                                            &object_info, 0);
      g_type_add_interface_static (object_type,
                                   FO_TYPE_NEUTRAL_FO,
                                   &fo_neutral_fo_info);
    }
  
  return object_type;
}

static void
fo_multi_properties_init (FoMultiProperties *fo_multi_properties)
{
}

static void
fo_multi_properties_class_init (FoMultiPropertiesClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_multi_properties_finalize;

  object_class->set_property = fo_multi_properties_set_property;
  object_class->get_property = fo_multi_properties_get_property;

  FO_FO_CLASS (klass)->debug_dump_properties = fo_multi_properties_debug_dump_properties;
  FO_FO_CLASS (klass)->update_from_context = fo_multi_properties_update_from_context;

  g_object_class_install_property
    (object_class,
     PROP_ID,
     g_param_spec_object ("id",
			  _("Id"),
			  _("Id property"),
			  FO_TYPE_PROPERTY,
			  G_PARAM_READABLE));
  g_object_class_install_property
    (object_class,
     PROP_ROLE,
     g_param_spec_object ("role",
			  _("Role"),
			  _("Role property"),
			  FO_TYPE_PROPERTY,
			  G_PARAM_READABLE));
  g_object_class_install_property
    (object_class,
     PROP_SOURCE_DOCUMENT,
     g_param_spec_object ("source-document",
			  _("Source Document"),
			  _("Source Document property"),
			  FO_TYPE_PROPERTY,
			  G_PARAM_READABLE));
}

static void
fo_multi_properties_finalize (GObject *object)
{
  FoMultiProperties *fo_multi_properties;

  fo_multi_properties = FO_MULTI_PROPERTIES (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


static void
fo_multi_properties_set_property (GObject         *object,
                         guint            prop_id,
                         const GValue    *value,
                         GParamSpec      *pspec)
{
  FoMultiProperties *fo_multi_properties;

  fo_multi_properties = FO_MULTI_PROPERTIES (object);

  switch (prop_id)
    {
    case PROP_ID:
      fo_multi_properties_set_id (fo_multi_properties, FO_PROPERTY_ID (g_value_get_object (value)));
      break;
    case PROP_ROLE:
      fo_multi_properties_set_role (fo_multi_properties, FO_PROPERTY_ROLE (g_value_get_object (value)));
      break;
    case PROP_SOURCE_DOCUMENT:
      fo_multi_properties_set_source_document (fo_multi_properties, FO_PROPERTY_SOURCE_DOCUMENT (g_value_get_object (value)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
fo_multi_properties_get_property (GObject         *object,
                         guint            prop_id,
                         GValue          *value,
                         GParamSpec      *pspec)
{
  FoMultiProperties *fo_multi_properties;

  fo_multi_properties = FO_MULTI_PROPERTIES (object);

  switch (prop_id)
    {
    case PROP_ID:
      g_value_set_object (value, G_OBJECT (fo_multi_properties_get_id (fo_multi_properties)));
      break;
    case PROP_ROLE:
      g_value_set_object (value, G_OBJECT (fo_multi_properties_get_role (fo_multi_properties)));
      break;
    case PROP_SOURCE_DOCUMENT:
      g_value_set_object (value, G_OBJECT (fo_multi_properties_get_source_document (fo_multi_properties)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_multi_properties_new:
 * 
 * Creates a new #FoMultiProperties initialized to default value.
 * 
 * Return value: the new #FoMultiProperties
 **/
FoFo*
fo_multi_properties_new (void)
{
  return FO_FO (g_object_new (fo_multi_properties_get_type (), NULL));
}

void
fo_multi_properties_update_from_context (FoFo *fo, FoContext *context)
{
  g_return_if_fail (fo != NULL);
  g_return_if_fail (FO_IS_MULTI_PROPERTIES (fo));

  fo_multi_properties_set_id (FO_MULTI_PROPERTIES (fo),
			  fo_context_get_id (context));
  fo_multi_properties_set_role (FO_MULTI_PROPERTIES (fo),
			  fo_context_get_role (context));
  fo_multi_properties_set_source_document (FO_MULTI_PROPERTIES (fo),
			  fo_context_get_source_document (context));
}

void
fo_multi_properties_debug_dump_properties (FoFo *fo, gint depth)
{
  FoMultiProperties *fo_multi_properties;

  g_return_if_fail (fo != NULL);
  g_return_if_fail (FO_IS_MULTI_PROPERTIES (fo));

  fo_multi_properties = FO_MULTI_PROPERTIES (fo);

  fo_object_debug_dump (fo_multi_properties->id, depth);
  fo_object_debug_dump (fo_multi_properties->role, depth);
  fo_object_debug_dump (fo_multi_properties->source_document, depth);

  FO_FO_CLASS (parent_class)->debug_dump_properties (fo, depth + 1);
}

/**
 * fo_multi_properties_set_id:
 * @fo_multi_properties: The @FoMultiProperties object
 * @new_id: The new "id" property value
 * 
 * Sets the #id property of @multi_properties to @new_id
 **/
void
fo_multi_properties_set_id (FoMultiProperties *fo_multi_properties,
		         FoPropertyId *new_id)
{
  g_return_if_fail (fo_multi_properties != NULL);
  g_return_if_fail (FO_IS_MULTI_PROPERTIES (fo_multi_properties));

  if (new_id)
    g_object_ref (G_OBJECT (new_id));
  if (fo_multi_properties->id)
    g_object_unref (G_OBJECT (fo_multi_properties->id));
  fo_multi_properties->id = FO_PROPERTY_ID (new_id);
  /*g_object_notify (G_OBJECT (fo_multi_properties), "id");*/
}

/**
 * fo_multi_properties_get_id:
 * @fo_multi_properties: The @FoMultiProperties object
 *
 * Gets the #id property of @multi_properties
 *
 * Return value: The "id" property value
**/
FoPropertyId*
fo_multi_properties_get_id (FoMultiProperties *fo_multi_properties)
{
  g_return_val_if_fail (fo_multi_properties != NULL, NULL);
  g_return_val_if_fail (FO_IS_MULTI_PROPERTIES (fo_multi_properties), NULL);

  return fo_multi_properties->id;
}

/**
 * fo_multi_properties_set_role:
 * @fo_multi_properties: The @FoMultiProperties object
 * @new_role: The new "role" property value
 * 
 * Sets the #role property of @multi_properties to @new_role
 **/
void
fo_multi_properties_set_role (FoMultiProperties *fo_multi_properties,
		         FoPropertyRole *new_role)
{
  g_return_if_fail (fo_multi_properties != NULL);
  g_return_if_fail (FO_IS_MULTI_PROPERTIES (fo_multi_properties));

  if (new_role)
    g_object_ref (G_OBJECT (new_role));
  if (fo_multi_properties->role)
    g_object_unref (G_OBJECT (fo_multi_properties->role));
  fo_multi_properties->role = FO_PROPERTY_ROLE (new_role);
  /*g_object_notify (G_OBJECT (fo_multi_properties), "role");*/
}

/**
 * fo_multi_properties_get_role:
 * @fo_multi_properties: The @FoMultiProperties object
 *
 * Gets the #role property of @multi_properties
 *
 * Return value: The "role" property value
**/
FoPropertyRole*
fo_multi_properties_get_role (FoMultiProperties *fo_multi_properties)
{
  g_return_val_if_fail (fo_multi_properties != NULL, NULL);
  g_return_val_if_fail (FO_IS_MULTI_PROPERTIES (fo_multi_properties), NULL);

  return fo_multi_properties->role;
}

/**
 * fo_multi_properties_set_source_document:
 * @fo_multi_properties: The @FoMultiProperties object
 * @new_source_document: The new "source-document" property value
 * 
 * Sets the #source-document property of @multi_properties to @new_source_document
 **/
void
fo_multi_properties_set_source_document (FoMultiProperties *fo_multi_properties,
		         FoPropertySourceDocument *new_source_document)
{
  g_return_if_fail (fo_multi_properties != NULL);
  g_return_if_fail (FO_IS_MULTI_PROPERTIES (fo_multi_properties));

  if (new_source_document)
    g_object_ref (G_OBJECT (new_source_document));
  if (fo_multi_properties->source_document)
    g_object_unref (G_OBJECT (fo_multi_properties->source_document));
  fo_multi_properties->source_document = FO_PROPERTY_SOURCE_DOCUMENT (new_source_document);
  /*g_object_notify (G_OBJECT (fo_multi_properties), "source-document");*/
}

/**
 * fo_multi_properties_get_source_document:
 * @fo_multi_properties: The @FoMultiProperties object
 *
 * Gets the #source-document property of @multi_properties
 *
 * Return value: The "source-document" property value
**/
FoPropertySourceDocument*
fo_multi_properties_get_source_document (FoMultiProperties *fo_multi_properties)
{
  g_return_val_if_fail (fo_multi_properties != NULL, NULL);
  g_return_val_if_fail (FO_IS_MULTI_PROPERTIES (fo_multi_properties), NULL);

  return fo_multi_properties->source_document;
}
